"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteUser = exports.updateUserData = exports.getLoggedUserData = exports.clearAuthData = exports.setAuthData = exports.createAuthDataProvider = exports.authDataProvider = exports.isTokenExpired = exports.authorizeWithRefreshToken = exports.authorizeWithUserPass = void 0;
const axios_1 = __importDefault(require("axios"));
const apiTools_1 = require("./apiTools");
const mutex_1 = require("./utils/mutex");
const jwt_decode_1 = require("jwt-decode");
let _authData = null;
const mutex = new mutex_1.Mutex(250);
function authorizeWithUserPass(baseUrl, tenant, basicAuthPass, username, password) {
    return __awaiter(this, void 0, void 0, function* () {
        let response = null;
        try {
            response = yield axios_1.default.post(`${baseUrl}/auth-oauth2/oauth/token`, { username, password, grant_type: 'password', scope: 'read' }, {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                    Authorization: `Basic ${basicAuthPass}`,
                    'X-Tenant': tenant,
                },
            });
            _authData = response.data;
            return true;
        }
        catch (error) {
            //console.error('Authorization error');
            //console.error(error);
            return false;
        }
    });
}
exports.authorizeWithUserPass = authorizeWithUserPass;
function authorizeWithRefreshToken(baseUrl, tenant, basicAuthPass, refreshToken) {
    return __awaiter(this, void 0, void 0, function* () {
        let response = null;
        try {
            response = yield axios_1.default.post(`${baseUrl}/auth-oauth2/oauth/token`, { refresh_token: refreshToken, grant_type: 'refresh_token' }, {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                    Authorization: `Basic ${basicAuthPass}`,
                    'X-Tenant': tenant,
                },
            });
            _authData = response.data;
            // console.log('authData', _authData);
            //console.log('REFRESH TOKEN AUTH RESULT ***');
            //console.log(_authData);
            return true;
        }
        catch (error) {
            //console.error('Authorization error');
            //console.error(error)
            return false;
        }
    });
}
exports.authorizeWithRefreshToken = authorizeWithRefreshToken;
function isTokenExpired(token) {
    // check if we can use current token
    if (token) {
        return (
        // 10s before actual expiry
        (0, jwt_decode_1.jwtDecode)(token).exp * 1000 < new Date().getTime() + 10000);
    }
    return true;
}
exports.isTokenExpired = isTokenExpired;
const authDataProvider = (ctx_1, refreshTokenStorageHandler_1, ...args_1) => __awaiter(void 0, [ctx_1, refreshTokenStorageHandler_1, ...args_1], void 0, function* (ctx, refreshTokenStorageHandler, forceRefresh = false) {
    // console.log( `authDataProvider invoked ----     currentToken=${  _authData ? _authData.access_token : '(null)'   }`, );
    if (!forceRefresh && _authData && _authData.access_token) {
        if (!isTokenExpired(_authData.access_token)) {
            return { token: _authData.access_token, UUID: _authData.UUID };
        }
    }
    //console.log('******** GET AUTH TOKEN (REMOTE CALL) ********');
    let loginSuccess = false;
    const refreshToken = yield refreshTokenStorageHandler.getRefreshToken(ctx.TENANT);
    if (refreshToken && !isTokenExpired(refreshToken)) {
        //console.log('*** AUTHORIZE WITH REFRESH TOKEN ');
        // use mutex to enforce max 1 api call at a time
        loginSuccess = yield mutex.getLock(() => {
            if (_authData && !isTokenExpired(_authData.access_token)) {
                return Promise.resolve(true);
            }
            return authorizeWithRefreshToken(ctx.BASE_URL, ctx.TENANT, ctx.BASIC_AUTH, refreshToken);
        });
    }
    else if (refreshToken) {
        refreshTokenStorageHandler.clearRefreshToken(ctx.TENANT);
    }
    if (!loginSuccess && ctx.anonymousAuth) {
        // console.log('**** ANONYMOUS AUTH ****');
        loginSuccess = yield authorizeWithUserPass(ctx.BASE_URL, ctx.TENANT, ctx.BASIC_AUTH, 'anonymous', '');
    }
    if (!loginSuccess || !_authData) {
        return { token: '', UUID: '' };
    }
    else {
        refreshTokenStorageHandler.setRefreshToken(ctx.TENANT, _authData.refresh_token);
        return { token: _authData.access_token, UUID: _authData.UUID };
    }
});
exports.authDataProvider = authDataProvider;
function createAuthDataProvider(ctx, refreshTokenProvider, forceRefresh = false) {
    return () => (0, exports.authDataProvider)(ctx, refreshTokenProvider, forceRefresh);
}
exports.createAuthDataProvider = createAuthDataProvider;
function setAuthData(tenant, newAuthData, refreshTokenStorageHandler) {
    _authData = newAuthData;
    refreshTokenStorageHandler.setRefreshToken(tenant, _authData.refresh_token);
}
exports.setAuthData = setAuthData;
function clearAuthData(tenant, refreshTokenStorageHandler) {
    refreshTokenStorageHandler.clearRefreshToken(tenant);
    _authData = null;
}
exports.clearAuthData = clearAuthData;
function getLoggedUserData(baseURL, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield (0, axios_1.default)({
            method: 'get',
            url: `${baseURL}/auth-api/api/me`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        }).catch(apiTools_1.handleAPIError);
        return response ? response.data : null;
    });
}
exports.getLoggedUserData = getLoggedUserData;
function updateUserData(baseURL, token, userData) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield axios_1.default
            .post(`${baseURL}/auth-api/api/me`, userData, {
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        })
            .catch(apiTools_1.handleAPIError);
        return (response && response.data) || false;
    });
}
exports.updateUserData = updateUserData;
function deleteUser(baseURL, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const { login } = yield getLoggedUserData(baseURL, token);
        const response = yield axios_1.default
            .request({
            url: `${baseURL}/auth-api/api/me`,
            method: 'DELETE',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data: { login },
        })
            .catch(apiTools_1.handleAPIError);
        return (response && response.data) || false;
    });
}
exports.deleteUser = deleteUser;
